#define _LCD_FIRST_ROW          0x80     // Move cursor to the 1st row
#define _LCD_SECOND_ROW         0xC0     // Move cursor to the 2nd row
#define _LCD_THIRD_ROW          0x94     // Move cursor to the 3rd row
#define _LCD_FOURTH_ROW         0xD4     // Move cursor to the 4th row
#define _LCD_CLEAR              0x01     // Clear display
#define _LCD_RETURN_HOME        0x02     // Return cursor to home position, returns a shifted display to
                                         // its original position. Display data RAM is unaffected.
#define _LCD_CURSOR_OFF         0x0C     // Turn off cursor
#define _LCD_UNDERLINE_ON       0x0E     // Underline cursor on
#define _LCD_BLINK_CURSOR_ON    0x0F     // Blink cursor on
#define _LCD_MOVE_CURSOR_LEFT   0x10     // Move cursor left without changing display data RAM
#define _LCD_MOVE_CURSOR_RIGHT  0x14     // Move cursor right without changing display data RAM
#define _LCD_TURN_ON            0x0C     // Turn Lcd display on
#define _LCD_TURN_OFF           0x08     // Turn Lcd display off
#define _LCD_SHIFT_LEFT         0x18     // Shift display left without changing display data RAM
#define _LCD_SHIFT_RIGHT        0x1E     // Shift display right without changing display data RAM

// LCD Definitions
#define LCD_ADDR                0x4E

const unsigned short HEX00=0x00;
const unsigned short HEX01=0x01;
const unsigned short HEX04=0x04;
const unsigned short HEX06=0x06;
const unsigned short HEX08=0x08;
const unsigned short HEX20=0x20;
const unsigned short HEX28=0x28;
const unsigned short HEX30=0x30;
const unsigned short HEX80=0x80;
const unsigned short HEX94=0x94;
const unsigned short HEXC0=0xC0;
const unsigned short HEXD4=0xD4;
const unsigned short HEXF0=0xF0;

void LCD_Cmd(char out_char) {
  char hi_n, lo_n;
  char rs = HEX00;

  hi_n = out_char & HEXF0;
  lo_n = (out_char << 4) & HEXF0;

  I2C1_Start();
  I2C1_Is_Idle();
  I2C1_Wr(LCD_ADDR);
  I2C1_Is_Idle();
  I2C1_Wr(hi_n | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(hi_n | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  Delay_50us();
  I2C1_Wr(lo_n | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(lo_n | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  I2C1_stop();

  if(out_char == HEX01)
    Delay_ms(2);
}

void LCD_Chr(char row, char column, char out_char) {
  char hi_n, lo_n;
  char rs = HEX01;

  column--;
  switch(row){
    case 1:
      LCD_Cmd(HEX80+column);
      break;
    case 2:
      LCD_Cmd(HEXC0+column);
      break;
    case 3:
      LCD_Cmd(HEX94+column);
      break;
    case 4:
      LCD_Cmd(HEXD4+column);
      break;
  };

  hi_n = out_char & HEXF0;
  lo_n = (out_char << 4) & HEXF0;

  I2C1_Start();
  I2C1_Is_Idle();
  I2C1_Wr(LCD_ADDR);
  I2C1_Is_Idle();
  I2C1_Wr(hi_n | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(hi_n | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  Delay_50us();
  I2C1_Wr(lo_n | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(lo_n | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  I2C1_stop();
}

void LCD_Chr_Cp(char out_char) {
  char hi_n, lo_n;
  char rs = HEX01;

  hi_n = out_char & HEXF0;
  lo_n = (out_char << 4) & HEXF0;

  I2C1_Start();
  I2C1_Is_Idle();
  I2C1_Wr(LCD_ADDR);
  I2C1_Is_Idle();
  I2C1_Wr(hi_n | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(hi_n | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  Delay_50us();
  I2C1_Wr(lo_n | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(lo_n | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  I2C1_stop();
}


void LCD_Init() {
  char rs = HEX00;

  I2C1_Start();
  I2C1_Is_Idle();
  I2C1_Wr(LCD_ADDR);
  I2C1_Is_Idle();

  Delay_10ms();
  Delay_10ms();
  Delay_10ms();

  I2C1_Wr(HEX30 | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(HEX30 | rs | HEX00 | HEX08);
  I2C1_Is_Idle();

  Delay_10ms();

  I2C1_Wr(HEX30 | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(HEX30 | rs | HEX00 | HEX08);
  I2C1_Is_Idle();

  Delay_10ms();

  I2C1_Wr(HEX30 | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_10ms();
  Delay_10ms();
  Delay_10ms();
  Delay_10ms();
  Delay_10ms();
  I2C1_Wr(HEX30 | rs | HEX00 | HEX08);
  I2C1_Is_Idle();

  Delay_10ms();

  I2C1_Wr(HEX20 | rs | HEX04 | HEX08);
  I2C1_Is_Idle();
  Delay_50us();
  I2C1_Wr(HEX20 | rs | HEX00 | HEX08);
  I2C1_Is_Idle();
  I2C1_Stop();

  Delay_10ms();

  LCD_Cmd(HEX28);
  LCD_Cmd(HEX06);
}

void LCD_Out(char row, char col, char *text) {
  while(*text)
    LCD_Chr(row, col++, *text++);
}

void LCD_Out_Cp(char *text) {
  while(*text)
    LCD_Chr_Cp(*text++);
}